#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the ThreadManager class."""

from threading import activeCount
from zope.interface import implements
from boduch.interface import IThreadManager
from boduch.event import EventThread

class ThreadManager:
    """The ThreadManager is responsible for starting new event threads.
    Event threads are threads where handles are executed.
    
    The threaded attribute indicates if threading is actually enabled.
    
    The max_threads attribute indicates how many threads are allowed to
    run concurrently."""
    implements(IThreadManager)
    threaded=False
    max_threads=10
    
    @classmethod
    def get_threaded(cls):
        """Return true if the thread manager is running in threaded mode."""
        return cls.threaded
    
    @classmethod
    def start_event_thread(cls, queue):
        """Start a new thread of control to run the first encountered handle
        in the specified queue."""
        if cls.max_threads>activeCount():
            EventThread(queue).run()
    
__all__=['ThreadManager']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the interface for the event thread class."""

from zope.interface import Interface, Attribute

class IEventThread(Interface):
    """The event thread class must store a list of handles to execute
    and must define a run method which actually executes the handles."""
    handles=Attribute("The handles that are executed by this thread.")
    
    def run(self):
        """Run the thread and in turn, execute all the handlers for the
        event."""
        
__all__=['IEventThread']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines some utility functions that are designed to make
using the event system slightly easier."""

from boduch.event import EventManager, ThreadManager

def publish(evt, args=[], kw={}, atomic=False):
    """Make a publish event call to the event manager."""
    EventManager.publish(evt, args=args, kw=kw, atomic=atomic)
    
def subscribe(evt, handle):
    """Make a subcribe to event call to the event manager."""
    EventManager.subscribe(evt, handle)
    
def unsubscribe(evt, handle):
    """Make a unsubscribe to event call to the event manager."""
    EventManager.unsubscribe(evt, handle)
    
def threaded(is_threaded):
    """Set the mode of the event manager.  True for threaded mode."""
    ThreadManager.threaded=is_threaded
    
__all__=['publish', 'subscribe', 'unsubscribe', 'threaded']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The event package.  The sub-modules defined in this package are all related
to implementing an event-based system."""

from boduch.event.eventthread import EventThread
from boduch.event.event import Event
from boduch.event.threadmanager import ThreadManager
from boduch.event.eventmanager import EventManager
from boduch.event.util import publish, subscribe, unsubscribe, threaded
from boduch.event.data import EventSetPush, EventSetPop, EventSetSort,\
                              EventSetGet, EventSetSet, EventHashPush,\
                              EventHashPop, EventHashGet

__all__=['Event', 'EventManager', 'EventThread',\
         'ThreadManager', 'publish', 'subscribe', 'unsubscribe', 'threaded',\
         'EventSetPush', 'EventSetPop', 'EventSetGet',\
         'EventSetSort', 'EventHashPush', 'EventHashPop', 'EventHashGet']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from zope.interface import Interface, Attribute

class IThreadManager(Interface):
    threaded=Attribute("""True if the thread manager is running in threaded
    mode.""")
    max_threads=Attribute("""Maximum number of threads that can run.""")
    
    def get_threaded(cls):
        """Return true if the thread manager is running in threaded mode."""
 
    def start_event_thread(cls, queue):
        """Start a new thread of control to run the first handle found in 
        the specified queue."""
        
__all__=['IThreadManager']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from boduch.handle import Handle
from boduch import PRIORITY_CRITICAL

class HandleHashPush(Handle):
    """Handle that actually places the data onto the hash in response to the
    EventHashPush event."""
    def __init__(self, *args, **kw):
        Handle.__init__(self, *args, **kw)
        self.priority=PRIORITY_CRITICAL
        
    def run(self):
        """Run the handle.  First we get the hash instance and the new pair
        from the event.  Next we acquire the event lock and perform the
        push."""
        event_obj=self.data['event']
        hash_obj=event_obj.data['hash']
        pair=event_obj.data['pair']
        hash_obj.data[pair[0]]=pair[1]
        
__all__=['HandleHashPush']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides a test case class for testing events called
TestEvent and a test suite which contains the event test case class."""

import unittest

from boduch.interface import IEvent
from boduch.event import Event, publish, subscribe, unsubscribe
from boduch.handle import Handle

class TestEvent(unittest.TestCase):
    """This class is derived from the TestCase class which is part of the
    builtin unittest Python module.  There are several methods defined by
    this class each representing some area of event testing."""
    def setUp(self):
        """This method is invoked before any tests are run.  Here we create
        a test event instance which is a attribute of the TestEvent class.
        This instance is then available to the remaining methods defined in
        this class."""
        self.test_event_obj=Event()
        
    def test_A_interface(self):
        """Perform some event interface tests.  First, we verify that the 
        Event class implements the IEvent interface and the test event 
        instance provides the IEvent interface."""
        self.assertTrue(IEvent.implementedBy(Event),\
                        'IEvent not implemented by Event.')
        self.assertTrue(IEvent.providedBy(self.test_event_obj),\
                        'IEvent not provided by Event instance.')
                        
    def test_B_subscribe(self):
        """Perform a subscribe test.  Here, we simply subscribe a basic
        handler to handle a basic event."""
        try:
            subscribe(Event, Handle)
        except Exception, e:
            self.fail(str(e))
            
    def test_C_publish(self):
        """Perform a publish test.  Here, we publish a simple event."""
        try:
            publish(Event)
        except NotImplementedError:
            pass
        except Exception, e:
            self.fail(str(e))
                        
SuiteEvent=unittest.TestLoader().loadTestsFromTestCase(TestEvent)

__all__=['TestEvent', 'SuiteEvent']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the ISet interface."""

from zope.interface import Interface, Attribute

class ISet(Interface):
    """The basic interface for set objects."""
    data=Attribute("The set data.")
    
    def __getitem__(self, key):
        """Allows retrieval of the specified key using Python list notation."""
        
    def __setitem__(self, key, value):
        """Allows setting items using Python list notation."""
        
    def __delitem__(self, key):
        """Allows deleting items using Python list notation."""
   
    def push(self, obj):
        """Push an object onto the set."""
        
    def pop(self, index):
        """Remove the object at the specified index from the set."""
        
    def get(self, index):
        """Return the object at the specified index."""
        
    def set(self, index, obj):
        """Update the specified index with the specified object."""
        
    def sort(self, reverse=False):
        """Sort the set data."""
        
__all__=['ISet']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the core event system functionality."""

from Queue import Queue
from zope.interface import implements
from boduch.interface import IEventManager
from boduch.event import ThreadManager
from boduch.type import is_type

class EventManager(object):
    """This class is the core event system.  This is an abstract class and
    there are therefore no instances.  This class defines two class attributes.
    
    The queue attribute is where Handle instances are placed when running in
    threaded mode.  Event threads then consume handles from the queue.
    
    The subscriptions attribute is a dictionary containing all event handlers.
    The key of this dictionary is the event class itself."""
    implements(IEventManager)
    subscriptions={}
    queue=Queue(0)
    
    @classmethod
    def get_subscriptions(cls, evt=None):
        """Return the handle instances that are subscribed to the specified
        event.  If the event is not specified, return all handles."""
        if evt and cls.subscriptions.has_key(evt):
            return cls.subscriptions[evt]
        return cls.subscriptions
    
    @classmethod
    def publish(cls, evt, args=[], kw={}, atomic=False):
        """Publish an event.
        
        The evt parameter is the event class to instantiate.  The args 
        parameter and the kw parameter are passed to the event constructor.
        
        If the atomic parameter is true, any subscribed to the event will
        execute synchronously in this thread."""
        if cls.get_subscriptions().has_key(evt):
            params={'event':evt(*args, **kw)}
            handles=cls.build_handlers(evt, params=params)
            if ThreadManager.get_threaded() and not atomic:
                for handle in handles:
                    cls.queue.put(handle)
                    ThreadManager.start_event_thread(cls.queue)
            else:
                for handle in handles:
                    handle.run()
        
    @classmethod
    def subscribe(cls, evt, handle):
        """Subscribe to an event.  The event parameter must inherit from Event
        and the handle parameter must inherit from Handle."""
        if not is_type(evt, "Event"):
            raise TypeError("Invalid event.")
        if not is_type(handle, "Handle"):
            raise TypeError("Invalid handle.")
        if cls.get_subscriptions().has_key(evt):
            if handle not in cls.get_subscriptions(evt=evt):
                cls.subscriptions[evt].append(handle)
                cls.prioritize(evt)
        else:
            cls.subscriptions[evt]=[handle]
            
    @classmethod
    def unsubscribe(cls, evt, handle):
        """Unsubcribe the specified handler from the specified event."""
        if cls.get_subscriptions().has_key(evt):
            cnt=0
            for handle_obj in cls.get_subscriptions(evt=evt):
                if handle==handle_obj:
                    cls.subscriptions[evt].pop(cnt)
                    return
                cnt+=1
                
    @classmethod
    def prioritize(cls, evt):
        """Prioritize the handlers for the specified event in descending
        order.  This means that handles that evaluate to a higher integer
        value, they will execute first for an given event."""
        cls.subscriptions[evt].sort(reverse=True)
            
    @classmethod
    def build_handlers(cls, evt, params={}):
        """Build and return a list of event handlers for the specified 
        event."""
        handlers=[]
        for handler in cls.get_subscriptions(evt=evt):
            handlers.append(handler(**params))
        return handlers
    
__all__=['EventManager']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the base type class.  This class defines some
operator-overloading functionality."""

import types
import uuid
from zope.interface import implements
from boduch.interface import IType

class Type(object):
    """The base type class that defines various comparison operators."""
    implements(IType)
    
    def __init__(self, *args, **kw):
        """Constructor.  Initialize the keyword data.  The data attribute
        is used when computing the length of type instances."""
        self.data=kw
        self.uuid=uuid.uuid1()
        
    def __len__(self):
        """Compute the length of this instance.  We iterate through the 
        values in the data attribute.  We then return the total of all
        integers found in the data attribute."""
        length=0
        for i in self.data.values():
            try:
                if type(i) is types.IntType:
                    length+=i
                    continue
                length+=len(i)
            except (TypeError, AttributeError):
                continue
        return length
    
    def __eq__(self, other):
        """Compare the equality of this instance to another.  We return true
        if the length of the two instances match"""
        return len(self)==len(other)
    
    def __lt__(self, other):
        """Return true if this instance is less than the other instance."""
        return len(self)<len(other)
    
    def __gt__(self, other):
        """Return true if this instance is greater than the other instance."""
        return len(self)>len(other)
    
    def __cmp__(self, other):
        """Return -1 if both instances have a priority attribute and this
        one is less than the other; 1 if both instances have a priority
        attribute and this one is greater than the other; 0 in all other
        cases."""
        if hasattr(self, 'priority') and hasattr(other, 'priority'):
            if self.priority<other.priority:
                return -1
            elif self.priority>other.priority:
                return 1
        return 0
    
__all__=['Type']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines constants that identify primitive types."""

TYPE_BOOL=       "BOOL"
TYPE_BOOLEAN=    "BOOLEAN"
TYPE_INT=        "INT"
TYPE_INTEGER=    "INTEGER"
TYPE_LONG=       "LONG"
TYPE_FLOAT=      "FLOAT"
TYPE_STR=        "STR"
TYPE_STRING=     "STRING"
TYPE_UNICODE=    "UNICODE"
TYPE_TUPLE=      "TUPLE"
TYPE_LIST=       "LIST"
TYPE_DICT=       "DICT"
TYPE_DICTIONARY= "DICTIONARY"

__all__=['TYPE_BOOL', 'TYPE_BOOLEAN', 'TYPE_INT', 'TYPE_INTEGER', 'TYPE_LONG',\
         'TYPE_FLOAT', 'TYPE_STR', 'TYPE_STRING', 'TYPE_UNICODE',\
         'TYPE_TUPLE', 'TYPE_LIST', 'TYPE_DICT', 'TYPE_DICTIONARY']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This package defines constants used in the boduch Python library."""

from boduch.constant.priority import PRIORITY_CRITICAL, PRIORITY_MAJOR,\
                                     PRIORITY_MINOR, PRIORITY_TRIVIAL
from boduch.constant.type import TYPE_BOOL, TYPE_BOOLEAN, TYPE_INT,\
                                 TYPE_INTEGER, TYPE_LONG, TYPE_FLOAT,\
                                 TYPE_STR, TYPE_STRING, TYPE_UNICODE,\
                                 TYPE_TUPLE, TYPE_LIST, TYPE_DICT,\
                                 TYPE_DICTIONARY
                                    
__all__=['PRIORITY_CRITICAL', 'PRIORITY_MAJOR', 'PRIORITY_MINOR',\
         'PRIORITY_TRIVIAL', 'TYPE_BOOL', 'TYPE_BOOLEAN', 'TYPE_INT',\
         'TYPE_INTEGER', 'TYPE_LONG', 'TYPE_FLOAT', 'TYPE_STR',\
         'TYPE_STRING', 'TYPE_UNICODE', 'TYPE_TUPLE', 'TYPE_LIST',\
         'TYPE_DICT', 'TYPE_DICTIONARY']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module contains a utility function for executing all the test
suites."""

import unittest

from boduch.test import SuiteEvent, SuiteSet, SuiteHash
from boduch.event import threaded

def run_tests(is_threaded=False):
    """This function creates a test suite, adds all other suites to it, and
    runs the tests."""
    threaded(is_threaded)
    MainTestSuite=unittest.TestSuite()
    MainTestSuite.addTest(SuiteEvent)
    MainTestSuite.addTest(SuiteSet)
    MainTestSuite.addTest(SuiteHash)
    unittest.TextTestRunner(verbosity=2).run(MainTestSuite)
    
__all__=['run_tests']


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides a test case for the Set class and a test
suite which contains a the test case class."""

import unittest

from boduch.interface import ISet
from boduch.event import Event, publish, subscribe, unsubscribe
from boduch.handle import Handle
from boduch.data import Set

class TestSet(unittest.TestCase):
    """This class is the test case for Set instances."""
    def setUp(self):
        """Create the test Set instance."""
        self.set_obj=Set()
        
    def test_A_interface(self):
        """Test the Set instaface."""
        self.assertTrue(ISet.implementedBy(Set),\
                        'ISet not implemented by Set.')
        self.assertTrue(ISet.providedBy(self.set_obj),\
                        'ISet not provided by Set instance.')
                        
    def test_B_push(self):
        """Test the Set.push() method."""
        try:
            self.set_obj.push('test')
        except Exception, e:
            self.fail(str(e))
            
    def test_C_sort(self):
        """Test the Set.sort() method."""
        try:
            self.set_obj.sort()
        except Exception, e:
            self.fail(str(e))
        
    def test_D_get(self):
        """Test the Set.get() method."""
        try:
            self.set_obj.get(0)
        except Exception, e:
            self.fail(str(e))
            
    def test_E_pop(self):
        """Test the Set.pop() method."""
        try:
            self.set_obj.pop(0)
        except Exception, e:
            self.fail(str(e))
            
    def test_F_setitem(self):
        """Test the setitem operator."""
        try:
            self.set_obj.push('test')
            self.set_obj[0]='testing'
        except Exception, e:
            self.fail(str(e))
            
    def test_G_getitem(self):
        """Test the getitem operator."""
        try:
            self.set_obj[0]
        except Exception, e:
            self.fail(str(e))
            
    def test_H_delitem(self):
        """Test the delitem operator."""
        try:
            del self.set_obj[0]
        except Exception, e:
            self.fail(str(e))
            
SuiteSet=unittest.TestLoader().loadTestsFromTestCase(TestSet)

__all__=['TestSet', 'SuiteSet']            
                        


#boduch - Simple Python tool library.
#   Copyright (C) 2008  Adam Boduch
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module defines the event thread class that is used to execute
event handlers."""

from threading import Thread
from Queue import Empty
from zope.interface import implements
from boduch.type import Type
from boduch.interface import IEventThread

class EventThread(Type, Thread):
    """This class extends the core thread class to start a new thread of
    control."""
    implements(IEventThread)

    def __init__(self, queue):
        """Constructor.  Initialize the thread class as well as the event
        queue."""
        Type.__init__(self)
        Thread.__init__(self, name=self.uuid)
        self.queue=queue
        
    def run(self):
        """Execute each event handler in a new thread."""
        while True:
            try:
                handle=self.queue.get_nowait()
                handle.run()
            except Empty:
                break
            
__all__=['EventThread']